/// HyperKycFlow Enum is used to define workflow
enum HyperKycFlow {
  document,
  face,
}

/// HyperKycConfig is used to launch HyperKyc
/// Create an instance of HyperKycConfig and call launch with it to start the workflow
class HyperKycConfig {
  /// AppID to launch HyperKYC : Get this from Hyperverge team
  final String appId;

  /// AccessToken to launch HyperKYC : Please contact Hyperverge team to understand how to generate HV accessToken
  final String accessToken;

  /// TransactionId to launch HyperKYC : Please contact Hyperverge team to understand more on sessions and transactionId
  final String transactionId;

  /// Workflow : The HyperKyc SDK allows its clients to pass in a custom ordered list of HyperKycFlow types to configure the order in which capture screens and other operations happen within the SDK.
  final List<HyperKycFlow> workFlow;

  /// Default Constructor to create HyperKycConfig instance
  HyperKycConfig({
    required this.appId,
    required this.accessToken,
    required this.transactionId,
    required this.workFlow,
  });

  /// Create HyperKycConfig using HyperKycConfigBuilder instance
  HyperKycConfig.fromBuilder(
      {required HyperKycConfigBuilder hyperKycConfigBuilder})
      : assert(
          hyperKycConfigBuilder._appId != null,
          'AppID cannot be null, Please use setAppID(appId) in HyperKycConfigBuilder',
        ),
        assert(
          hyperKycConfigBuilder._accessToken != null,
          'AccessToken cannot be null, Please use setAccessToken(accessToken) in HyperKycConfigBuilder',
        ),
        assert(
          hyperKycConfigBuilder._transactionId != null,
          'TransactionId cannot be null, Please use setTransactionId(transactionId) in HyperKycConfigBuilder',
        ),
        assert(
          hyperKycConfigBuilder._workFlow != null,
          'WorkFlow cannot be null, Please use setWorkFlow(workFlow) in HyperKycConfigBuilder',
        ),
        assert(
            hyperKycConfigBuilder._workFlow!.contains(HyperKycFlow.face) &&
                hyperKycConfigBuilder._workFlow!
                    .contains(HyperKycFlow.document),
            'Please make sure you add both HyperKycFlow.document and HyperKycFlow.face flows in the workflow'),
        appId = hyperKycConfigBuilder._appId!,
        accessToken = hyperKycConfigBuilder._accessToken!,
        transactionId = hyperKycConfigBuilder._transactionId!,
        workFlow = hyperKycConfigBuilder._workFlow!;

  /// Overriden toString() method
  @override
  String toString() {
    return 'HyperKycConfig{appId: $appId, accessToken: $accessToken, transactionId: $transactionId, workFlow: $workFlow}';
  }

  /// Convert HyperKycConfig object to Map
  Map<String, dynamic> toMap() {
    return {
      'appId': this.appId,
      'accessToken': this.accessToken,
      'transactionId': this.transactionId,
      'workFlow': this.workFlow.map((e) => e.name).join(','),
    };
  }

  /// Convert Map to HyperKycConfig object
  factory HyperKycConfig.fromMap(Map<dynamic, dynamic> map) {
    return HyperKycConfig(
      appId: map['appId'] as String,
      accessToken: map['accessToken'] as String,
      transactionId: map['transactionId'] as String,
      workFlow: map['workFlow'] as List<HyperKycFlow>,
    );
  }
}

/// Builder class to build HyperKycConfig
class HyperKycConfigBuilder {
  /// AppID to launch HyperKYC : Get this from Hyperverge team
  String? _appId;

  /// AccessToken to launch HyperKYC : Please contact Hyperverge team to understand how to generate HV accessToken
  String? _accessToken;

  /// TransactionId to launch HyperKYC : Please contact Hyperverge team to understand more on sessions and transactionId
  String? _transactionId;

  /// Workflow : The HyperKyc SDK allows its clients to pass in a custom ordered list of HyperKycFlow types to configure the order in which capture screens and other operations happen within the SDK.
  List<HyperKycFlow>? _workFlow;

  /// Default Constructor to create HyperKycConfigBuilder instance
  HyperKycConfigBuilder();

  /// static HyperKycConfigBuilder instance
  static final HyperKycConfigBuilder hyperKycConfigBuilder =
      HyperKycConfigBuilder();

  /// Use this setter to set HyperKycConfig appId
  HyperKycConfigBuilder setAppId({required String appId}) {
    hyperKycConfigBuilder._appId = appId;
    return hyperKycConfigBuilder;
  }

  /// Use this setter to set HyperKycConfig accessToken
  HyperKycConfigBuilder setAccessToken({required String accessToken}) {
    hyperKycConfigBuilder._accessToken = accessToken;
    return hyperKycConfigBuilder;
  }

  /// Use this setter to set HyperKycConfig transactionId
  HyperKycConfigBuilder setTransactionId({required String transactionId}) {
    hyperKycConfigBuilder._transactionId = transactionId;
    return hyperKycConfigBuilder;
  }

  /// Use this setter to set HyperKycConfig workFlow
  HyperKycConfigBuilder setWorkFlow({required List<HyperKycFlow> workFlow}) {
    hyperKycConfigBuilder._workFlow = workFlow;
    return hyperKycConfigBuilder;
  }

  /// Use this build function to create HyperKycConfig instance from builder properties
  HyperKycConfig build() {
    return HyperKycConfig.fromBuilder(
        hyperKycConfigBuilder: hyperKycConfigBuilder);
  }

  /// Overriden toString() method
  @override
  String toString() {
    return 'HyperKycConfigBuilder{_appId: $_appId, _accessToken: $_accessToken, _transactionId: $_transactionId, _workFlow: $_workFlow}';
  }
}
